/*
 *
 *  MULti Core Operating System - MultiLayer Kernel (MULCOS-M)
 *                      ITRON Kernel header
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       kernel.h
 *
 *  Mar/06/2024     Created.
 *
 */


#ifndef KERNEL_H
#define KERNEL_H

#include "itron.h"
#include "arch_config.h"
#include "mulcosx.h"

/*
 *      Definitions of package environment
 */

#define TKERNEL_PRID_MAKER  0x1000


/*
 *      Definitions of packet
 */

typedef struct t_ctsk {
    ATR         tskatr;         /* Attribute                                */
    VP_INT      exinf;          /* Extended information                     */
    FP          task;           /* Start address                            */
    PRI         itskpri;        /* Initial task priority                    */
    SIZE        stksz;          /* Stack size                               */
    VP          stk;            /* Stack address                            */
    SIZE        sstksz;         /* Secure stack size (only non-secure task) */
    VB const    *name;          /* Task name                                */
} T_CTSK;

typedef struct t_rtsk {
    STAT        tskstat;        /* Task status                              */
    PRI         tskpri;         /* Task current priority                    */
    PRI         tskbpri;        /* Task base priority                       */
    STAT        tskwait;        /* Waiting factor                           */
    ID          wobjid;         /* Object ID for waiting                    */
    TMO         lefttmo;        /* Left time                                */
    UINT        actcnt;         /* Activate request count                   */
    UINT        wupcnt;         /* Wake up request count                    */
    UINT        suscnt;         /* Suspendhing request count                */
    VB          name[16];       /* Task name                                */
} T_RTSK;

typedef struct t_rtst {
    STAT        tskstat;        /* Task status                              */
    STAT        tskwait;        /* Waiting factor                           */
} T_RTST;

typedef struct t_csem {
    ATR         sematr;         /* Attribute                                */
    UINT        isemcnt;        /* Initial semaphore count                  */
    UINT        maxsem;         /* Max semaphore count                      */
    VB const    *name;          /* Semaphore name                           */
} T_CSEM;

typedef struct t_rsem {
    ID          wtskid;         /* Waiting task ID of semaphore             */
    UINT        semcnt;         /* Current semaphore count                  */
    VB          name[16];       /* Semaphore name                           */
} T_RSEM;

typedef struct t_cflg {
    ATR         flgatr;         /* Attribute                                */
    FLGPTN      iflgptn;        /* Initial event flag pattern               */
    VB const    *name;          /* Event flag name                          */
} T_CFLG;

typedef struct t_rflg {
    ID          wtskid;         /* Waiting task ID of event flag            */
    FLGPTN      flgptn;         /* Current event flag pattern               */
    VB          name[16];       /* Event flag name                          */
} T_RFLG;

typedef struct t_pflg {
    FLGPTN      waiptn;         /* Wait bit pattern                         */
    MODE        wfmode;         /* Waitng mode                              */
    FLGPTN     *flgptn;         /* Bit pattern when waiting is released     */
    TMO         tmout;          /* Specified timeout                        */
} T_PFLG;

typedef struct t_cdtq {
    ATR         dtqatr;         /* Attribute                                */
    UINT        dtqcnt;         /* Data element count                       */
    VP          dtq;            /* Address of data queue                    */
    VB const    *name;          /* Data queue name                          */
} T_CDTQ;

typedef struct t_rdtq {
    ID          stskid;         /* Waiting task ID of send-wait             */
    ID          rtskid;         /* Waiting task ID of receive-wait          */
    UINT        sdtqcnt;        /* Data count of data queue                 */
    VB          name[16];       /* Data queue name                          */
} T_RDTQ;

typedef struct t_msg {
    struct t_msg    *msgque;    /* Message queue header                     */
} T_MSG;

typedef struct t_msg_pri {
    T_MSG       msgque;         /* Message queue                            */
    PRI         msgpri;         /* Message priority                         */
} T_MSG_PRI;

typedef struct t_cmbx {
    ATR         mbxatr;         /* Attribute                                */
    PRI         mbxmpri;        /* Max message priority                     */
    VP          mprihd;         /* Base address of message queue            */
    UINT        msgmax;         /* Maximum number of messages               */
    VB const    *name;          /* Mail box name                            */
} T_CMBX;

typedef struct t_rmbx {
    ID          wtskid;         /* Waiting task ID of mail box              */
    T_MSG       *pk_msg;        /* Top mail address of mail box             */
    UINT        msgcnt;         /* Message count in mail box                */
    VB          name[16];       /* Mail box name                            */
} T_RMBX;

typedef struct t_cmbf {
    ATR         mbfatr;         /* Attribute                                */
    UINT        maxmsz;         /* Maximum message size                     */
    SIZE        mbfsz;          /* Size of message buffer area              */
    VP          mbf;            /* Startr address of message buffer area    */
    VB const    *name;          /* Message buffer name                      */
} T_CMBF;

typedef struct t_rmbf {
    ID          stskid;         /* Waiting task ID of send-wait             */
    ID          rtskid;         /* Waiting task ID of receive-wait          */
    UINT        smsgcnt;        /* The number of messages in message buffer */
    SIZE        fmbfsz;         /* Size of free message buffer              */
    VB          name[16];       /* Message buffer name                      */
} T_RMBF;

typedef struct t_cmtx {
    ATR         mtxatr;         /* Attribute                                */
    PRI         ceilpri;        /* Ceiling priority                         */
    VB const    *name;          /* Mutex name                               */
} T_CMTX;

typedef struct t_rmtx {
    ID          htskid;         /* Locking task ID of mutex                 */
    ID          wtskid;         /* Waiting task ID of lock-wait             */
    VB          name[16];       /* Mutex name                               */
} T_RMTX;

typedef struct t_cmpf {
    ATR         mpfatr;         /* Attribute                                */
    UINT        blkcnt;         /* Memory block count                       */
    UINT        blksz;          /* Memory block size                        */
    VP          mpf;            /* Base address of memory pool area         */
    VB const    *name;          /* Memory pool name                         */
} T_CMPF;

typedef struct t_rmpf {
    ID          wtskid;         /* Waiting task ID of memory pool           */
    UINT        fblkcnt;        /* Free memory block count                  */
    VB          name[16];       /* Memory pool name                         */
} T_RMPF;

typedef struct t_cmpl {
    ATR         mplatr;         /* Attribute                                */
    SIZE        mplsz;          /* Size of memory pool area                 */
    VP          mpl;            /* Base address of memory pool area         */
    UINT        blkmax;         /* Maximum number of memory block           */
    VB const    *name;          /* Memory pool name                         */
} T_CMPL;

typedef struct t_rmpl {
    ID          wtskid;         /* Waiting task ID of memory pool           */
    SIZE        fmplsz;         /* Total size of free memory blocks         */
    UINT        fblksz;         /* Maximum memory blocksize available       */
    VB          name[16];       /* Memory pool name                         */
} T_RMPL;

typedef struct t_ccyc {
    ATR         cycatr;         /* Attribute                                */
    VP_INT      exinf;          /* Externel information                     */
    FP          cychdr;         /* Cyclic handler address                   */
    RELTIM      cyctim;         /* Cyclic time of activation                */
    RELTIM      cycphs;         /* Phase time of activation                 */
    VB const    *name;          /* Cyclic handler name                      */
} T_CCYC;

typedef struct t_rcyc {
    STAT        cycstat;        /* Cyclic handler status                    */
    RELTIM      lefttim;        /* Cyclic handler Left time                 */
    RELTIM      cyctim;         /* Cyclic time of activation                */
    VP_INT      exinf;          /* Externel information                     */
    VB          name[16];       /* Cyclic handler name                      */
} T_RCYC;

typedef struct t_dinh {
    ATR         inhatr;         /* Attribute                                */
    FP          inthdr;         /* Start address                            */
    IMASK       imask;          /* Interrupt level                          */
} T_DINH;

typedef struct t_cisr {
    ATR         isratr;         /* Attribute                                */
    VP_INT      exinf;          /* Externel information                     */
    INTNO       intno;          /* Interrupt number to ISR                  */
    FP          isr;            /* Interrupt service routine address        */
    IMASK       imask;          /* Interrupt level                          */
    VB const    *name;          /* Cyclic handler name                      */
} T_CISR;

typedef struct t_risr {
    INTNO       intno;          /* Interrupt number to ISR                  */
    FP          isr;            /* Interrupt service routine address        */
    VB          name[16];       /* Cyclic handler name                      */
} T_RISR;

typedef struct t_rsys {
    SIZE        free_sysmem;    /* Free size of system memory               */
    SIZE        free_usrmem;    /* Free size of user's memory               */
    UINT        used_task;      /* Num of in use task                       */
    UINT        used_sem;       /* Num of in use Semafore                   */
    UINT        used_eflg;      /* Num of in use eventflag                  */
    UINT        used_mbox;      /* Num of in use mail box                   */
    UINT        used_dtq;       /* Num of in use data queue                 */
    UINT        used_mtex;      /* Num of in use mutex                      */
    UINT        used_mbf;       /* Num of in use message buffer             */
    UINT        used_mpf;       /* Num of in use fixed-size memory pool     */
    UINT        used_mpl;       /* Num of in use variable-size memory pool  */
    UINT        used_isr;       /* Num of in use interrupt service routine  */
    UINT        used_cyc;       /* Num of in use cyclic handler             */
} T_RSYS;

typedef struct t_rcfg {
    SIZE        sysmem_size;    /* Size of system memory                    */
    SIZE        usrmem_size;    /* Size of user's memory                    */
    UINT        max_task;       /* Max task ID                              */
    UINT        max_sem;        /* Max Semafore ID                          */
    UINT        max_eflg;       /* Max eventflag ID                         */
    UINT        max_mbox;       /* Max mail box ID                          */
    UINT        max_dtq;        /* Max data queue ID                        */
    UINT        max_mtex;       /* Max mutex ID                             */
    UINT        max_mbf;        /* Max message bufferID                     */
    UINT        max_mpf;        /* Max fixed-size memory pool ID            */
    UINT        max_mpl;        /* Max variable-size memory pool ID         */
    UINT        max_isr;        /* Max interrupt service routine ID         */
    UINT        max_cyc;        /* Max cyclic handler ID                    */
} T_RCFG;

typedef struct t_rver {
    UH          maker;          /* Maker's code                             */
    UH          prid;           /* Product ID                               */
    UH          spver;          /* Specification version                    */
    UH          prver;          /* Kernel version                           */
    UH          prno[4];        /* Management information                   */
} T_RVER;


/*
 *      Definitios of function type
 */

#define TASK    void

/*
 *      Definitions of object attribute
 */

#define TA_NULL     0           /* No attribute                             */
#define TA_HLNG     0x00u       /* Hi-level language interface              */
#define TA_ASM      0x01u       /* Assembly language interface              */
#define TA_ACT      0x02u       /* Activate task                            */
#define TA_VFP      0x10u       /* Enable floating-point unit               */
#define TA_PRIV     0x20u       /* Privileged execution TASK                */

#define TA_TFIFO    0x00u       /* Task waiting queue is in FIFO order      */
#define TA_TPRI     0x01u       /* Task waiting queue is in task priority order */
#define TA_INHERIT  0x02u       /* Mutex use in priority inheritance protocol */
#define TA_CEILING  0x03u       /* Mutex use in priority criling protocol   */

#define TA_MFIFO    0x00u       /* Message queue is in FIFO order           */
#define TA_MPRI     0x02u       /* Message queue is in message priority order */

#define TMO_POL     0           /* Polling                                  */
#define TMO_FEVR    -1          /* Forever waiting                          */

#define TA_WSGL     0x00u       /* Only one task is allowed to waiting      */
#define TA_WMUL     0x02u       /* Multiple tasks are allowed to waiting    */
#define TA_CLR      0x04u       /* Eventflag's bit pattern is cleared       */

#define TWF_ANDW    0x00u       /* And waiting of event flag                */
#define TWF_ORW     0x01u       /* Or waiting of event flag                 */

#define TA_STA      0x02u       /* Start cyclic handler                     */
#define TA_PHS      0x04u       /* Phase lock                               */


/*
 *      Definitions Kernel Configuration
 */

#define TMIN_TPRI   1           /* Minimum task priority                    */
#define TMAX_TPRI   MULCOSX_MAX_TPRI    /* Maximum task priority            */

#define TMIN_MPRI   1           /* Minimum message priority                 */
#define TMAX_MPRI   MULCOSX_MAX_MPRI    /* Maximum message priority         */

#define TMAX_ACTCNT MULCOSX_MAX_ACT     /* Maximum activation request count */
#define TMAX_WUPCNT MULCOSX_MAX_WUP     /* Maximum wakeup request count     */
#define TMAX_SUSCNT MULCOSX_MAX_SUS     /* Maximum suspension count         */

#define TMAX_MAXSEM MAX_SEM_COUNT       /* Maximum semphore count           */

#define TMAX_TMO    2000000     /* Maximum TMO value                        */
#define TMAX_RELTIM 4000000     /* Maximum RELTIM value                     */

#define TBIT_FLGPTN 32          /* Number of bits in eventflag              */

#define MSEC2USEC   1000        /* Definition timerate (1 msec = 1000 usec) */
                                /* Definition of conversion of error code   */
#define ERR2E(p1)   ((int32_t)((int8_t)p1))


/*
 *      Definitions of required memory size
 */

#define TSZ_DTQ(n)    ((SIZE)((n)*sizeof(VP)))
#define TSZ_MPF(n,s)  ((SIZE)((s)*(n)))
#define TSZ_MPL(i,j)  ((i)*((j+MEMORY_MPL_ALIGN-1)&(0-MEMORY_MPL_ALIGN)))


/*
 *      Definitions of object status
 */

#define TTS_RUN     0x01u       /* Running state                            */
#define TTS_RDY     0x02u       /* Ready state                              */
#define TTS_WAI     0x04u       /* Waiting state                            */
#define TTS_SUS     0x08u       /* Suspended state                          */
#define TTS_WAS     0x0Cu       /* Waiting-suspended state                  */
#define TTS_DMT     0x10u       /* Dormant state                            */

#define TTW_SLP     0x0001u     /* Sleeping state                           */
#define TTW_DLY     0x0002u     /* Delayed state                            */
#define TTW_SEM     0x0004u     /* Waing state for semaphore                */
#define TTW_FLG     0x0008u     /* Waitng sate for event flag               */
#define TTW_SDTQ    0x0010u     /* Sending waiting sate for data queue      */
#define TTW_RDTQ    0x0020u     /* Receiving wating state for data queue    */
#define TTW_MBX     0x0040u     /* Receiving waiting state for mailbox      */
#define TTW_MTX     0x0080u     /* Waiting for mutex                        */
#define TTW_SMBF    0x0100u     /* Sending waiting state for message buffer */
#define TTW_RMBF    0x0200u     /* Receiving waiting state for messege buffe*/
#define TTW_MPF     0x2000u     /* Waitng sate for fixed-size memory pool   */
#define TTW_MPL     0x4000u     /* Waitng sate for variable-size memory pool*/
#define TTW_SLP2    0x8000u   /* Sleeping state 2 (externel spec)         */

#define TCYC_STP    0x00u       /* Operating state                          */
#define TCYC_STA    0x01u       /* Non-operating state                      */

#define TSK_SELF    0           /* Specifying invoking task                 */
#define TSK_NONE    0           /* No applicable task                       */

#define TPRI_SELF   0           /* Base priority of the invoking task       */
#define TPRI_INI    0           /* Initial priority of the invoking task    */


/*
 *      Definitions of error code
 */

#define E_SYS       -5          /* 0xFFFB: System error                     */
#define E_NOSPT     -9          /* 0xFFF7: Non-support error                */
#define E_RSATR     -11         /* 0xFFF5: Reserved attribute               */
#define E_PAR       -17         /* 0xFFEF: Parameter error                  */
#define E_ID        -18         /* 0xFFEE: Invalid ID error                 */
#define E_CTX       -25         /* 0xFFE7: Contex error                     */
#define E_ILUSE     -28         /* 0xFFE4: Invalid use error                */
#define E_NOMEM     -33         /* 0xFFDF: Memory empty error               */
#define E_NOID      -34         /* 0xFFDE: ID empty error                   */
#define E_OBJ       -41         /* 0xFFD7: Object status error              */
#define E_NOEXS     -42         /* 0xFFD6: Non-exist object error           */
#define E_QOVR      -43         /* 0xFFD5: Queing overflow error            */
#define E_RLWAI     -49         /* 0xFFCF: Release waiting                  */
#define E_TMOUT     -50         /* 0xFFCE: Time out error                   */
#define E_DLT       -51         /* 0xFFCD: Deletet object error             */


/*
 *      Definitions of version information
 */

#define TKERNEL_MAKER   0x0000  /* Kernel maker code                        */
                                /* Identification number of kernel          */
#define TKERNEL_PRID    (TKERNEL_PRID_MAKER|TKERNEL_PRID_ARCH)
#define TKERNEL_SPVER   0x5403  /* Version number of ITRON specification    */
#define TKERNEL_PPVER   0x0100  /* Version number of kernel                 */


/*
 *      Definitions of system call API
 */

ER_ID   acre_tsk(T_CTSK *pk_ctsk);
ER      cre_tsk(ID tskid, T_CTSK *pk_ctsk);
ER      del_tsk(ID tskid);
ER      act_tsk(ID tskid);
ER      iact_tsk(ID tskid);
ER_UINT can_act(ID tskid);
ER      sta_tsk(ID tskid, VP_INT stacd);
void    ext_tsk(void);
void    exd_tsk(void);
ER      ter_tsk(ID tskid);
ER      chg_pri(ID tskid, PRI tskpri);
ER      get_pri(ID tskid, PRI *p_tskpri);
ER      ref_tsk(ID tskid, T_RTSK *pk_rtsk);
ER      ref_tst(ID tskid, T_RTST *pk_rtst);
#define acre_tsk(p1)            (cre_tsk(0,p1))
#define iact_tsk(p1)            (act_tsk(p1))

ER      slp_tsk(void);
ER      tslp_tsk(TMO timout);
ER      wup_tsk(ID tskid);
ER      iwup_tsk(ID tskid);
ER_UINT can_wup(ID tskid);
ER      rel_wai(ID tskid);
ER      irel_wai(ID tskid);
ER      sus_tsk(ID tskid);
ER      rsm_tsk(ID tskid);
ER      frsm_tsk(ID tskid);
ER      dly_tsk(RELTIM dlytim);
#define slp_tsk()               (tslp_tsk(TMO_FEVR))
#define iwup_tsk(p1)            (wup_tsk(p1))
#define irel_wai(p1)            (rel_wai(p1))


ER_ID   acre_sem(T_CSEM *pk_csem);
ER      cre_sem(ID semid, T_CSEM *pk_csem);
ER      del_sem(ID semid);
ER      sig_sem(ID semid);
ER      isig_sem(ID semid);
ER      pol_sem(ID semid);
ER      wai_sem(ID semid);
ER      twai_sem(ID semid, TMO tmout);
ER      ref_sem(ID semid, T_RSEM *pk_rsem);
#define acre_sem(p1)            (cre_sem(0,p1))
#define isig_sem(p1)            (sig_sem(p1))
#define pol_sem(p1)             (twai_sem(p1,TMO_POL))
#define wai_sem(p1)             (twai_sem(p1,TMO_FEVR))


ER_ID   acre_flg(T_CFLG *pk_cflg);
ER      cre_flg(ID flgid, T_CFLG *pk_cflg);
ER      del_flg(ID flgid);
ER      set_flg(ID flgid, FLGPTN setptn);
ER      iset_flg(ID flgid, FLGPTN setptn);
ER      clr_flg(ID flgid, FLGPTN clrptn);
ER      wai_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn);
ER      pol_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn);
#if ((__ARM_ARCH == 8) && defined(__BUILD_TRZ__))
ER      vwai_flg(ID flgid, T_PFLG *pk_pflg);
static inline __attribute__((__always_inline__)) ER twai_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn, TMO tmout)
{
    T_PFLG pflg;

    pflg.waiptn = waiptn;
    pflg.wfmode = wfmode;
    pflg.flgptn = p_flgptn;
    pflg.tmout  = tmout;
    return vwai_flg(flgid, &pflg);
}
#else
ER      twai_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn, TMO tmout);
#endif
ER      ref_flg(ID flgid, T_RFLG *pk_rflg);
#define acre_flg(p1)            (cre_flg(0,p1))
#define iset_flg(p1,p2)         (set_flg(p1,p2))
#define pol_flg(p1,p2,p3,p4)    (twai_flg(p1,p2,p3,p4,TMO_POL))
#define wai_flg(p1,p2,p3,p4)    (twai_flg(p1,p2,p3,p4,TMO_FEVR))


ER_ID   acre_dtq(T_CDTQ *pk_cdtq);
ER      cre_dtq(ID dtqid, T_CDTQ *pk_cdtq);
ER      del_dtq(ID dtqid);
ER      psnd_dtq(ID dtqid, VP_INT data);
ER      ipsnd_dtq(ID dtqid, VP_INT data);
ER      snd_dtq(ID dtqid, VP_INT data);
ER      tsnd_dtq(ID dtqid, VP_INT data, TMO tmout);
ER      fsnd_dtq(ID dtqid, VP_INT data);
ER      ifsnd_dtq(ID dtqid, VP_INT data);
ER      prcv_dtq(ID dtqid, VP_INT *p_data);
ER      rcv_dtq(ID dtqid, VP_INT *p_data);
ER      trcv_dtq(ID dtqid, VP_INT *p_data, TMO tmout);
ER      ref_dtq(ID dtqid, T_RDTQ *pk_rdtq);
#define acre_dtq(p1)            (cre_dtq(0,p1))
#define ipsnd_dtq(p1,p2)        (tsnd_dtq(p1,p2,TMO_POL))
#define psnd_dtq(p1,p2)         (tsnd_dtq(p1,p2,TMO_POL))
#define snd_dtq(p1,p2)          (tsnd_dtq(p1,p2,TMO_FEVR))
#define ifsnd_dtq(p1,p2)        (fsnd_dtq(p1,p2))
#define prcv_dtq(p1,p2)         (trcv_dtq(p1,p2,TMO_POL))
#define rcv_dtq(p1,p2)          (trcv_dtq(p1,p2,TMO_FEVR))


ER_ID   acre_mbx(T_CMBX *pk_cmbx);
ER      cre_mbx(ID mbxid, T_CMBX *pk_cmbx);
ER      del_mbx(ID mbxid);
ER      snd_mbx(ID mbxid, T_MSG *pk_msg);
ER      snd_mbx2(ID mbxid, PRI msgpri, VP pk_msg);
ER      prcv_mbx(ID mbxid, T_MSG **ppk_msg);
ER      rcv_mbx(ID mbxid, T_MSG **ppk_msg);
ER      trcv_mbx(ID mbxid, T_MSG **ppk_msg, TMO tmout);
ER      prcv_mbx2(ID mbxid, VP *ppk_msg);
ER      rcv_mbx2(ID mbxid, VP *ppk_msg);
ER      trcv_mbx2(ID mbxid, VP *ppk_msg, TMO tmout);
ER      ref_mbx(ID mbxid, T_RMBX *pk_rmbx);
#define acre_mbx(p1)            (cre_mbx(0,p1))
#define prcv_mbx(p1,p2)         (trcv_mbx(p1,p2,TMO_POL))
#define rcv_mbx(p1,p2)          (trcv_mbx(p1,p2,TMO_FEVR))
#define prcv_mbx2(p1,p2)        (trcv_mbx(p1,p2,TMO_POL))
#define rcv_mbx2(p1,p2)         (trcv_mbx(p1,p2,TMO_FEVR))


ER_ID   acre_mbf(T_CMBF *pk_cmbf);
ER      cre_mbf(ID mbfid, T_CMBF *pk_cmbf);
ER      del_mbf(ID mbfid);
ER      psnd_mbf(ID mbfid, VP msg, UINT msgsz);
ER      snd_mbf(ID mbfid, VP msg, UINT msgsz);
ER      tsnd_mbf(ID mbfid, VP msg, UINT msgsz, TMO tmout);
ER_UINT prcv_mbf(ID mbfid, VP msg);
ER_UINT rcv_mbf(ID mbfid, VP msg);
ER_UINT trcv_mbf(ID mbfid, VP msg, TMO tmout);
ER      ref_mbf(ID mbfid, T_RMBF *pk_rmbf);
#define acre_mbf(p1)            (cre_mbf(0,p1))
#define psnd_mbf(p1,p2,p3)      (tsnd_mbf(p1,p2,p3,TMO_POL))
#define snd_mbf(p1,p2,p3)       (tsnd_mbf(p1,p2,p3,TMO_FEVR))
#define prcv_mbf(p1,p2)         (trcv_mbf(p1,p2,TMO_POL))
#define rcv_mbf(p1,p2)          (trcv_mbf(p1,p2,TMO_FEVR))


ER_ID   acre_mtx(T_CMTX *pk_cmtx);
ER      cre_mtx(ID mtxid, T_CMTX *pk_cmtx);
ER      del_mtx(ID mtxid);
ER      unl_mtx(ID mtxid);
ER      loc_mtx(ID mtxid);
ER      ploc_mtx(ID mtxid);
ER      tloc_mtx(ID mtxid, TMO tmout);
ER      ref_mtx(ID mtxid, T_RMTX *pk_rmtx);
#define acre_mtx(p1)            (cre_mtx(0,p1))
#define ploc_mtx(p1)            (tloc_mtx(p1,TMO_POL))
#define loc_mtx(p1)             (tloc_mtx(p1,TMO_FEVR))


ER_ID   acre_mpf(T_CMPF *pk_cmpf);
ER      cre_mpf(ID mpfid, T_CMPF *pk_cmpf);
ER      del_mpf(ID mpfid);
ER      pget_mpf(ID mpfid, VP *p_blk);
ER      get_mpf(ID mpfid, VP *p_blk);
ER      tget_mpf(ID mpfid, VP *p_blk, TMO tmout);
ER      rel_mpf(ID mpfid, VP p_blk);
ER      ref_mpf(ID mpfid, T_RMPF *pk_rmpf);
#define acre_mpf(p1)            (cre_mpf(0,p1))
#define pget_mpf(p1,p2)         (tget_mpf(p1,p2,TMO_POL))
#define get_mpf(p1,p2)          (tget_mpf(p1,p2,TMO_FEVR))


ER_ID   acre_mpl(T_CMPL *pk_cmpl);
ER      cre_mpl(ID mplid, T_CMPL *pk_cmpl);
ER      del_mpl(ID mplid);
ER      pget_mpl(ID mplid, UINT blksz, VP *p_blk);
ER      get_mpl(ID mplid, UINT blksz, VP *p_blk);
ER      tget_mpl(ID mplid, UINT blksz, VP *p_blk, TMO tmout);
ER      rel_mpl(ID mplid, VP p_blk);
ER      ref_mpl(ID mplid, T_RMPL *pk_rmpl);
#define acre_mpl(p1)            (cre_mpl(0,p1))
#define pget_mpl(p1,p2,p3)      (tget_mpl(p1,p2,p3,TMO_POL))
#define get_mpl(p1,p2,p3)       (tget_mpl(p1,p2,p3,TMO_FEVR))


ER_ID   acre_cyc(T_CCYC *pk_ccyc);
ER      cre_cyc(ID cycid, T_CCYC *pk_ccyc);
ER      del_cyc(ID cycid);
ER      sta_cyc(ID cycid);
ER      stp_cyc(ID cycid);
ER      ref_cyc(ID cycid, T_RCYC *pk_rcyc);
#define acre_cyc(p1)            (cre_cyc(0,p1))


ER      isig_tim(void);
ER      set_tim(SYSTIM *p_systim);
ER      get_tim(SYSTIM *p_systim);
ER      rot_rdq(PRI tskpri);
ER      irot_rdq(PRI tskpri);
ER      get_tid(ID *p_tskid);
ER      iget_tid(ID *p_tskid);
ER      loc_cpu(void);
ER      iloc_cpu(void);
ER      unl_cpu(void);
ER      iunl_cpu(void);
ER      dis_dsp(void);
ER      ena_dsp(void);
ER      ref_sys(T_RSYS *pk_rsys);
ER      ref_cfg(T_RCFG *pk_rcfg);
ER      ref_ver(T_RVER *pk_rver);
BOOL    sns_ctx(void);
BOOL    sns_loc(void);
BOOL    sns_dsp(void);
BOOL    sns_dpn(void);
#define irot_rdq(p1)            (rot_rdq(p1))
#define iget_tid(p1)            (get_tid(p1))
#define iloc_cpu()              (loc_cpu())
#define iunl_cpu()              (unl_cpu())


ER_ID   acre_isr(T_CISR *);
ER      cre_isr(ID isrid, T_CISR *pk_cisr);
ER      del_isr(ID isrid);
ER      ref_isr(ID isrid, T_RISR *pk_risr);
ER      def_inh(INHNO inhno, T_DINH *pk_dinh);
ER      ena_int(INTNO intno);
ER      dis_int(INTNO intno);
ER      chg_ims(UINT imask);
ER      get_ims(UINT *imask);
#define acre_isr(p1)            (cre_isr(0,p1))


/*
 *      Definitions of system call from secure layer of non-secure task
 */

#if ((__ARM_ARCH == 8) && defined(__BUILD_TRZ__) && defined(__BUILD_SECURE__))
ER      vset_psp(ID tskid, SIZE stksz);


ER      vn_act_tsk(ID tskid);
ER_UINT vn_can_act(ID tskid);
ER      vn_sta_tsk(ID tskid, VP_INT stacd);
ER      vn_ter_tsk(ID tskid);
ER      vn_ref_tsk(ID tskid, T_RTSK *pk_rtsk);
ER      vn_ref_tst(ID tskid, T_RTST *pk_rtst);

ER      vn_wup_tsk(ID tskid);
ER_UINT vn_can_wup(ID tskid);
ER      vn_rel_wai(ID tskid);
ER      vn_sus_tsk(ID tskid);
ER      vn_rsm_tsk(ID tskid);
ER      vn_frsm_tsk(ID tskid);

ER      vn_clr_flg(ID flgid, FLGPTN clrptn);
ER      vn_set_flg(ID flgid, FLGPTN setptn);
ER      vn_pol_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn);
ER      vn_ref_flg(ID flgid, T_RFLG *pk_rflg);

ER      vn_sig_sem(ID semid);
ER      vn_pol_sem(ID semid);
ER      vn_ref_sem(ID semid, T_RSEM *pk_rsem);

ER      vn_psnd_dtq(ID dtqid, VP_INT data);
ER      vn_fsnd_dtq(ID dtqid, VP_INT data);
ER      vn_prcv_dtq(ID dtqid, VP_INT *p_data);
ER      vn_ref_dtq(ID dtqid, T_RDTQ *pk_rdtq);

ER      vn_snd_mbx(ID mbxid, T_MSG *pk_msg);
ER      vn_snd_mbx2(ID mbxid, PRI msgpri, VP pk_msg);
ER      vn_prcv_mbx(ID mbxid, T_MSG **ppk_msg);
ER      vn_ref_mbx(ID mbxid, T_RMBX *pk_rmbx);
#define vn_prcv_mbx2(p1,p2)     (vn_prcv_mbx(p1,(T_MSG **)p2))

ER      vn_psnd_mbf(ID mbfid, VP msg, UINT msgsz);
ER_UINT vn_prcv_mbf(ID mbfid, VP msg);
ER      vn_ref_mbf(ID mbfid, T_RMBF *pk_rmbf);

ER      vn_pget_mpf(ID mpfid, VP *p_blk);
ER      vn_rel_mpf(ID mpfid, VP p_blk);
ER      vn_ref_mpf(ID mpfid, T_RMPF *pk_rmpf);

ER      vn_pget_mpl(ID mpfid, UINT blksz, VP *p_blk);
ER      vn_rel_mpl(ID mplid, VP p_blk);
ER      vn_ref_mpl(ID mplid, T_RMPL *pk_rmpl);

ER      vn_ref_sys(T_RSYS *pk_rsys);
#endif

/*
 *      Definitions of original system call API
 */

typedef struct t_csys {
    void      (*sysdwn)(UINT);  /* System down routine address              */
    void      (*sysidl)(void);  /* Idling routine address                   */
    UINT        usrmem_addr;    /* User memory area address                 */
    UINT        usrmem_size;    /* User memory area size                    */
    UINT        max_task;       /* Max task ID                              */
    UINT        rsv_task;       /* Num of reserved ID for manual assignment */
    UINT        max_sem;        /* Max Semafore ID                          */
    UINT        rsv_sem;        /* Num of reserved ID for manual assignment */
    UINT        max_eflg;       /* Max eventflag ID                         */
    UINT        rsv_eflg;       /* Num of reserved ID for manual assignment */
    UINT        max_mbox;       /* Max mail box ID                          */
    UINT        rsv_mbox;       /* Num of reserved ID for manual assignment */
    UINT        max_dtq;        /* Max data queue ID                        */
    UINT        rsv_dtq;        /* Num of reserved ID for manual assignment */
    UINT        max_mtex;       /* Max mutex ID                             */
    UINT        rsv_mtex;       /* Num of reserved ID for manual assignment */
    UINT        max_mbf;        /* Max message buffer ID                    */
    UINT        rsv_mbf;        /* Num of reserved ID for manual assignment */
    UINT        max_mpf;        /* Max fixed-size memory pool ID            */
    UINT        rsv_mpf;        /* Num of reserved ID for manual assignment */
    UINT        max_mpl;        /* Max variable-size memory pool ID         */
    UINT        rsv_mpl;        /* Num of reserved ID for manual assignment */
    UINT        max_isr;        /* Max interrupt service routine ID         */
    UINT        rsv_isr;        /* Num of reserved ID for manual assignment */
    UINT        max_cyc;        /* Max cyclic handler ID                    */
    UINT        rsv_cyc;        /* Num of reserved ID for manual assignment */
    UINT        max_task_pri;   /* Max task priority                        */
    UINT        clk_msec;       /* Tick time (usec)                         */
    INT         sgi_int_pri;    /* Exception priority of dispacher          */
    INT         sgi_no;         /* Exception number of dispacher            */
    T_CONFIG_ARCH     arch;     /* Dependent on architecture                */
} T_CSYS;


ER      sta_nsc(VP vect);
ER      ini_krn(T_CSYS *pk_csys);
void    sta_krn(void);

#endif
